import java.util.*;

public class BoardC implements Board {

    private Counter counter;
    private java.util.Random rand;
    private int[][]a;

    public BoardC(Counter counter) {
        this.counter = counter;

        rand = new java.util.Random();

        a = new int[3][3];
        for(int r = 0; r < 3; r++)
            for(int c = 0; c < 3; c++)
            a[r][c] = 0;
    }

    public void makeMove(Move move, boolean white) { 
        a[move.getRow()][move.getCol()] = white ? 1 : 2; 
    }
    public void unMakeMove(Move move) { 
        a[move.getRow()][move.getCol()] = 0; 
    }

    public boolean gameOver() {
        for(int i = 0; i < 3; i++)
            if((a[i][0] != 0) && ((a[i][0] == a[i][1] && a[i][0] == a[i][2]) || 
                                  (a[0][i] == a[1][i] && a[0][i] == a[2][i]))) return true;
    
        if((a[1][1] != 0) && ((a[0][0] == a[1][1] && a[0][0] == a[2][2]) || 
                              (a[0][2] == a[1][1] && a[1][1] == a[2][0]))) return true;
        return false;
    }

    private Moves getLegalMoves() {
        Moves answer = new MovesC();
        for(int i = 0; i < 3; i++) {
            for(int j = 0; j < 3; j++) {
                if(a[i][j] == 0)
                    //      if(Moves.DEBUG)
                    // System.out.println("getLegalMoves: adding move = " + new Move(i, j, counter).toString() + " done.");
                    answer.add(new MoveC(i, j, counter));
                }
        }
        return answer;
    }



    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("\n[");
        for(int r = 0; r < 3; r++) {
            sb.append("[");
            for(int c = 0; c < 3; c++)
                sb.append(a[r][c] + " ");
            sb.append("]\n ");
        }
        sb.append("]");
        return sb.toString() + "\n scoring " + this.sum();
    }
 
    private int sum() {
        int sum = 0;
        for(int r = 0; r < 3; r++)
            for(int c = 0; c < 3; c++)
                sum = sum + a[r][c];
        return sum;
    }
    
    private int score() {
        int sum = this.sum();
        if(false)
            System.out.println("score: scoring board = " + this.toString() + "\nscore: score = " + sum);

        return sum; 
    }

    public Move miniMax(int depth, boolean white) {

        Moves moves = this.getLegalMoves();

        if (depth == 1) {
            for(Move move : moves) {
                this.makeMove(move, white);
                move.setScore(this.score());
                this.unMakeMove(move);
            }
        }
        else
            this.manyMiniMax(moves, depth - 1, !white);
        
        return white ? moves.max() : moves.min();
    }

    private void manyMiniMax(Moves moves, int depth, boolean white) {
        
        for(Move move : moves) {
            this.makeMove(move, white);
            Move m = this.miniMax(depth, white);
            move.setScore(m.getScore());
            this.unMakeMove(move);
        }
    }
}